@@ -55,7 +55,7 @@ typedef struct decoder_state_s *DECODER_STATE;
 
 struct ber_decoder_s {
   AsnNode module;    /* the ASN.1 structure */
-  KsbaReader reader;
+  ksba_reader_t reader;
   const char *last_errdesc; /* string with the error description */
   int non_der;    /* set if the encoding is not DER conform */
   AsnNode root;   /* of the expanded parse tree */
@@ -91,10 +91,11 @@ new_decoder_state (void)
   ds->stacksize = 100;
   ds->idx = 0;
   ds->cur.node = NULL;
+  ds->cur.went_up = 0;
   ds->cur.in_seq_of = 0;
+  ds->cur.in_any = 0;
   ds->cur.again = 0;
   ds->cur.next_tag = 0;
-  ds->cur.went_up = 0;
   ds->cur.length = 0;
   ds->cur.ndef_length = 1;
   ds->cur.nread = 0;
@@ -158,21 +159,24 @@ set_error (BerDecoder d, AsnNode node, const char *text)
   fprintf (stderr,"ber-decoder: node `%s': %s\n", 
            node? node->name:"?", text);
   d->last_errdesc = text;
-  return KSBA_BER_Error;
+  return gpg_error (GPG_ERR_BAD_BER);
 }
 
 
 static int
 eof_or_error (BerDecoder d, int premature)
 {
-  if (ksba_reader_error (d->reader))
+  gpg_error_t err;
+
+  err = ksba_reader_error (d->reader);
+  if (err)
     {
       set_error (d, NULL, "read error");
-      return KSBA_Read_Error;
+      return err;
     }
   if (premature)
     return set_error (d, NULL, "premature EOF");
-  return -1;
+  return gpg_error (GPG_ERR_EOF);
 }
 
 static const char *
@@ -232,7 +236,9 @@ dump_tlv (const struct tag_info *ti, FILE *fp)
              ti->class == CLASS_APPLICATION? "APPLICATION" :
              ti->class == CLASS_CONTEXT? "CONTEXT-SPECIFIC" : "PRIVATE",
              ti->tag);
-  fprintf (fp, " %c hdr=%u len=", ti->is_constructed? 'c':'p', ti->nhdr);
+  fprintf (fp, " %c hdr=%lu len=",
+           ti->is_constructed? 'c':'p',
+           (unsigned long)ti->nhdr);
   if (ti->ndef)
     fputs ("ndef", fp);
   else
@@ -310,26 +316,26 @@ _ksba_ber_decoder_release (BerDecoder d)
  * 
  * Return value: 0 on success or an error code
  **/
-KsbaError
-_ksba_ber_decoder_set_module (BerDecoder d, KsbaAsnTree module)
+gpg_error_t
+_ksba_ber_decoder_set_module (BerDecoder d, ksba_asn_tree_t module)
 {
   if (!d || !module)
-    return KSBA_Invalid_Value;
+    return gpg_error (GPG_ERR_INV_VALUE);
   if (d->module)
-    return KSBA_Conflict; /* module already set */
+    return gpg_error (GPG_ERR_CONFLICT); /* module already set */
 
   d->module = module->parse_tree;
   return 0;
 }
 
 
-KsbaError
-_ksba_ber_decoder_set_reader (BerDecoder d, KsbaReader r)
+gpg_error_t
+_ksba_ber_decoder_set_reader (BerDecoder d, ksba_reader_t r)
 {
   if (!d || !r)
-    return KSBA_Invalid_Value;
+    return gpg_error (GPG_ERR_INV_VALUE);
   if (d->reader)
-    return KSBA_Conflict; /* reader already set */
+    return gpg_error (GPG_ERR_CONFLICT); /* reader already set */
   
   d->reader = r;
   return 0;
@@ -340,8 +346,9 @@ _ksba_ber_decoder_set_reader (BerDecoder d, KsbaReader r)
  ***********  decoding machinery  *************
  **********************************************/
 
+/* Read one byte, return that byte or -1 on error or EOF. */
 static int
-read_byte (KsbaReader reader)
+read_byte (ksba_reader_t reader)
 {
   unsigned char buf;
   size_t nread;
@@ -353,10 +360,10 @@ read_byte (KsbaReader reader)
   return rc? -1: buf;
 }
 
-/* read COUNT bytes into buffer.  buffer may be NULL to skip over
-   COUNT bytes.  Return 0 on success */
+/* Read COUNT bytes into buffer.  BUFFER may be NULL to skip over
+   COUNT bytes.  Return 0 on success or -1 on error. */
 static int 
-read_buffer (KsbaReader reader, char *buffer, size_t count)
+read_buffer (ksba_reader_t reader, char *buffer, size_t count)
 {
   size_t nread;
 
@@ -698,7 +705,7 @@ match_der (AsnNode root, const struct tag_info *ti,
 }
 
 
-static KsbaError 
+static gpg_error_t 
 decoder_init (BerDecoder d, const char *start_name)
 {
   d->ds = new_decoder_state ();
@@ -722,12 +729,12 @@ decoder_deinit (BerDecoder d)
 }
 
 
-static KsbaError
+static gpg_error_t
 decoder_next (BerDecoder d)
 {
   struct tag_info ti;
   AsnNode node;
-  KsbaError err;
+  gpg_error_t err;
   DECODER_STATE ds = d->ds;
   int debug = d->debug;
 
@@ -752,7 +759,7 @@ decoder_next (BerDecoder d)
           d->image.used = 0;
           d->image.buf = xtrymalloc (d->image.length);
           if (!d->image.buf)
-            return KSBA_Out_Of_Core;
+            return gpg_error (GPG_ERR_ENOMEM);
         }
 
       if (ti.nhdr + d->image.used >= d->image.length)
@@ -782,7 +789,7 @@ decoder_next (BerDecoder d)
                 {
                   /* We must push back the stuff we already read */
                   ksba_reader_unread (d->reader, ti.buf, ti.nhdr);
-                  return -1; 
+                  return gpg_error (GPG_ERR_EOF); 
                 }
               else
                 d->bypass = 1;
@@ -893,7 +900,7 @@ decoder_next (BerDecoder d)
   return 0;
 }
 
-static KsbaError
+static gpg_error_t
 decoder_skip (BerDecoder d)
 {
   if (d->val.primitive)
@@ -932,17 +939,17 @@ distance (AsnNode root, AsnNode node)
  * 
  * Return value: 
  **/
-KsbaError
+gpg_error_t
 _ksba_ber_decoder_dump (BerDecoder d, FILE *fp)
 {
-  KsbaError err;
+  gpg_error_t err;
   int depth = 0;
   AsnNode node;
   unsigned char *buf = NULL;
   size_t buflen = 0;;
 
   if (!d)
-    return KSBA_Invalid_Value;
+    return gpg_error (GPG_ERR_INV_VALUE);
 
   d->debug = !!getenv("DEBUG_BER_DECODER");
   d->use_image = 0;
@@ -977,13 +984,13 @@ _ksba_ber_decoder_dump (BerDecoder d, FILE *fp)
               buflen = d->val.length + 100;
               buf = xtrymalloc (buflen);
               if (!buf)
-                err = KSBA_Out_Of_Core;
+                err = gpg_error (GPG_ERR_ENOMEM);
             }
 
           for (n=0; !err && n < d->val.length; n++)
             {
               if ( (c=read_byte (d->reader)) == -1)
-                err =  eof_or_error (d, 1);
+                err = eof_or_error (d, 1);
               buf[n] = c;
             }
           if (err)
@@ -1018,7 +1025,7 @@ _ksba_ber_decoder_dump (BerDecoder d, FILE *fp)
         break;
 
     }
-  if (err == -1)
+  if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
 
   decoder_deinit (d);
@@ -1029,19 +1036,19 @@ _ksba_ber_decoder_dump (BerDecoder d, FILE *fp)
 
 
 
-KsbaError
+gpg_error_t
 _ksba_ber_decoder_decode (BerDecoder d, const char *start_name,
                           AsnNode *r_root,
                           unsigned char **r_image, size_t *r_imagelen)
 {
-  KsbaError err;
+  gpg_error_t err;
   AsnNode node;
   unsigned char *buf = NULL;
   size_t buflen = 0;
   unsigned long startoff;
 
   if (!d)
-    return KSBA_Invalid_Value;
+    return gpg_error (GPG_ERR_INV_VALUE);
 
   if (r_root)
     *r_root = NULL;
@@ -1093,13 +1100,13 @@ _ksba_ber_decoder_decode (BerDecoder d, const char *start_name,
               buflen = d->val.length + 100;
               buf = xtrymalloc (buflen);
               if (!buf)
-                err = KSBA_Out_Of_Core;
+                err = gpg_error (GPG_ERR_ENOMEM);
             }
 
           for (n=0; !err && n < d->val.length; n++)
             {
               if ( (c=read_byte (d->reader)) == -1)
-                err =  eof_or_error (d, 1);
+                err = eof_or_error (d, 1);
               buf[n] = c;
             }
           if (err)
@@ -1119,7 +1126,7 @@ _ksba_ber_decoder_decode (BerDecoder d, const char *start_name,
       if (err)
         break;
     }
-  if (err == -1)
+  if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
 
   if (r_root && !err)
@@ -1128,7 +1135,7 @@ _ksba_ber_decoder_decode (BerDecoder d, const char *start_name,
         { /* Not even the first node available - return eof */
           /* Fixme: release d->root */
           d->root = NULL;
-          err = -1;
+          err = gpg_error (GPG_ERR_EOF);
         }
       
       fixup_type_any (d->root);
